@@ -2,8 +2,8 @@
 Test toro.Queue.
 
 There are three sections, one each for tests that are
-1. adapted from Gevent's test_queue.py, except for FailingQueueTest of which I
-   don't understand the purpose,
+1. adapted from Gevent's test_queue.py, except for FailingQueueTest which
+    isn't applicable
 2. adapted from Gevent's test__queue.py,
 3. written specifically for Toro.
 """
@@ -34,52 +34,38 @@ class _TriggerTask(object):
         self.args = args
         self.kwargs = kwargs
         self.startedEvent = toro.Event()
+        self.completedEvent = toro.Event()
         IOLoop.instance().add_timeout(time.time() + 0.01, self.run)
 
     def run(self):
         self.startedEvent.set()
         self.fn(*self.args, **self.kwargs)
+        self.completedEvent.set()
 
 
 class QueueTest1(unittest.TestCase):
     type2test = toro.Queue
 
     @gen.engine
-    def do_blocking_test(self, block_func, block_args, block_kwargs, trigger_func, trigger_args, trigger_kwargs, callback):
+    def do_blocking_test(
+        self,
+        block_func, block_args, block_kwargs,
+        trigger_func, trigger_args, trigger_kwargs,
+        callback
+    ):
         self.t = _TriggerTask(trigger_func, trigger_args, trigger_kwargs)
         self.result = yield Task(block_func, *block_args, **block_kwargs)
-        # If block_func returned before our thread made the call, we failed!
+        # If block_func returned before our task made the call, we failed!
         if not self.t.startedEvent.is_set():
             self.fail("blocking function '%r' appeared not to block" %
                       block_func)
+
+        # make sure the task completes
+        yield Task(self.t.completedEvent.wait)
         callback(self.result)
 
     do_blocking_test.__test__ = False # Hide from nose
 
-    # Call this instead if block_func is supposed to raise an exception.
-    # TODO: useful?
-    def do_exceptional_blocking_test(self,block_func, block_args, trigger_func,
-                                   trigger_args, expected_exception_class):
-        self.t = _TriggerThread(trigger_func, trigger_args)
-        self.t.start()
-        try:
-            try:
-                block_func(*block_args)
-            except expected_exception_class:
-                raise
-            else:
-                self.fail("expected exception of kind %r" %
-                                 expected_exception_class)
-        finally:
-            self.t.join(10) # make sure the thread terminates
-            if self.t.isAlive():
-                self.fail("trigger function '%r' appeared to not return" %
-                                 trigger_func)
-            if not self.t.startedEvent.is_set():
-                self.fail("trigger thread ended but event never set")
-
-    do_exceptional_blocking_test.__test__ = False # Hide from nose
-
     @gen.engine
     def simple_queue_test(self, q, callback):
         if not q.empty():
@@ -98,22 +84,22 @@ class QueueTest1(unittest.TestCase):
             q.put(i)
             self.assert_(not q.empty(), "Queue should not be empty")
         self.assert_(not q.full(), "Queue should not be full")
-        q.put("last")
+        q.put(444)
         self.assert_(q.full(), "Queue should be full")
         try:
-            q.put("full")
+            q.put(555)
             self.fail("Didn't appear to block with a full queue")
         except Full:
             pass
 
-        # False is passed to the put() callback if it times out
+        # Full is passed to the put() callback if it times out
         self.assertEqual(Full, (yield Task(
-            q.put, "full", deadline=timedelta(seconds=0.01))))
+            q.put, 555, deadline=timedelta(seconds=0.01))))
         self.assertEquals(q.qsize(), QUEUE_SIZE)
         # Test a blocking put
-        yield Task(self.do_blocking_test, q.put, ("full",), {}, q.get, (), {})
+        yield Task(self.do_blocking_test, q.put, (555,), {}, q.get, (), {})
         yield Task(self.do_blocking_test, q.put,
-            ("full",), {'deadline': timedelta(seconds=10)}, q.get, (), {})
+            (555,), {'deadline': timedelta(seconds=10)}, q.get, (), {})
         # Empty it
         for i in range(QUEUE_SIZE):
             q.get()
@@ -127,9 +113,9 @@ class QueueTest1(unittest.TestCase):
         self.assertEqual(Empty, (yield Task(
             q.get, deadline=timedelta(seconds=0.01))))
         # Test a blocking get
-        yield Task(self.do_blocking_test, q.get, (), {}, q.put, ('empty',), {})
+        yield Task(self.do_blocking_test, q.get, (), {}, q.put, (555,), {})
         yield Task(self.do_blocking_test, q.get,
-            (), {'deadline': timedelta(seconds=10)}, q.put, ('empty',), {})
+            (), {'deadline': timedelta(seconds=10)}, q.put, (555,), {})
         callback()
 
     simple_queue_test.__test__ = False # Hide from nose
