@@ -1,9 +1,3 @@
-# TODO: check against API at http://www.gevent.org/gevent.queue.html#module-gevent.queue
-#   and emulate the example at its bottom in my docs
-# TODO: check on Gevent's licensing
-# TODO: review reprs and __str__'s
-# TODO: did I omit Gevent tests from the 2.7/ dir?
-# TODO: Py 3, test Jython & PyPy & add to classifiers
 from __future__ import with_statement
 
 import heapq
@@ -16,7 +10,7 @@ from tornado import stack_context
 from tornado.ioloop import IOLoop
 
 
-version_tuple = (0, 1)
+version_tuple = (0, 2)
 
 version = '.'.join(map(str, version_tuple))
 """Current version of Toro."""
@@ -36,10 +30,13 @@ __all__ = [
 
 
 class NotReady(Exception):
+    """Raised when accessing an :class:`AsyncResult` that has no value yet."""
     pass
 
 
 class AlreadySet(Exception):
+    """Raised when setting a value on an :class:`AsyncResult` that already
+    has one."""
     pass
 
 
@@ -60,9 +57,8 @@ class ToroBase(object):
         self.io_loop = io_loop or IOLoop.instance()
 
     def _next_tick(self, callback, *args, **kwargs):
-        if callback:
-            _check_callback(callback)
-            self.io_loop.add_callback(partial(callback, *args, **kwargs))
+        _check_callback(callback)
+        self.io_loop.add_callback(partial(callback, *args, **kwargs))
 
     def _consume_expired_waiters(self, waiters):
         # Delete waiters at the head of the queue who've timed out
@@ -449,7 +445,6 @@ class Queue(ToroBase):
             assert not self.queue, "queue non-empty, why are getters waiting?"
             getter = self.getters.popleft()
 
-            # Call _put and _get in case subclasses have special logic for them
             self._put(item)
             getter.run(self._get())
             self._run_callback(callback, True)
@@ -632,10 +627,9 @@ class Semaphore(object):
     If not given, value defaults to 1.
 
     .. note:: Unlike the standard threading.Semaphore_, a :class:`Semaphore`
-      can tell you the current value of its :attr:`counter` or whether it
-      is :meth:`locked`, because code in a single-threaded Tornado app can check
-      these values and act upon them without fear of interruption from another
-      thread.
+      can tell you the current value of its :attr:`counter`, because code in a
+      single-threaded Tornado app can check these values and act upon them
+      without fear of interruption from another thread.
 
     .. _threading.Semaphore: http://docs.python.org/library/threading.html#threading.Semaphore
 
@@ -656,9 +650,15 @@ class Semaphore(object):
         if value:
             self._unlocked.set()
 
+    def __repr__(self):
+        return '<%s at %s%s>' % (type(self).__name__, hex(id(self)), self._format())
+
     def __str__(self):
-        return '<%s counter=%s>' % (
-            self.__class__.__name__, self.counter)
+        return '<%s%s>' % (
+            self.__class__.__name__, self._format())
+
+    def _format(self):
+        return ' counter=%s' % self.counter
 
     @property
     def counter(self):
@@ -674,10 +674,9 @@ class Semaphore(object):
         on :meth:`acquire`.
         """
         self.q.put(None)
-        # TODO: what if locked() is still True here, because there was a
-        #   waiter for get() or because get() triggered a function that did
-        #   another acquire()? This needs a lot of testing.
-        self._unlocked.set()
+        if not self.locked():
+            # No one was waiting on acquire(), so self.q.qsize() is positive
+            self._unlocked.set()
 
     def wait(self, callback, deadline=None):
         """Wait for :attr:`locked` to be False
@@ -698,18 +697,14 @@ class Semaphore(object):
         """If a callback is passed, then decrement :attr:`counter` and run
         the callback. If the counter is zero, wait for
         a :meth:`release` or a timeout before running the callback.
+        If `deadline` is not ``None``, then the callback is passed ``False`` if
+        it times out before another coroutine calls :meth:`release`.
 
         If no callback is passed, then if the counter is zero return ``False``,
         else if the counter is positive decrement it and return ``True``.
 
-        .. note:: If you set a timeout, you can determine whether
-           `callback` was run because of a :meth:`release` or a timeout by
-           checking :meth:`locked`.
-
-        .. todo:: Correct?
-
         :Parameters:
-          - `callback`: Optional function taking no arguments.
+          - `callback`: Optional function taking one argument.
           - `deadline`: Optional timeout, either an absolute timestamp
             (as returned by ``time.time()``) or a ``datetime.timedelta`` for a
             deadline relative to the current time.
@@ -765,6 +760,10 @@ class Lock(object):
       thread has grabbed the lock, provided you do not yield to the IOLoop
       between checking :meth:`locked` and using a protected resource.
 
+    .. note:: This class is not a context manager, and cannot be used in a
+      with-statement the way Python's standard Lock can, because with-statements
+      don't work with callbacks.
+
     .. _threading.Lock: http://docs.python.org/library/threading.html#threading.Lock
 
     :Parameters:
@@ -812,4 +811,3 @@ class Lock(object):
     def locked(self):
         """``True`` if the lock has been acquired"""
         return self._block.locked()
-
